Package: mrtsSphere
Type: Package
Title: Multi-Resolution Thin-Plate Splines on the Sphere
Version: 0.1.2
Authors@R: c(
    person(given = "Hao-Yun", family = "Huang", role = c("aut", "cre"),
           email = "hhuscout@gmail.com",
           comment = c(ORCID = "0009-0004-8384-3261")),
    person(given = "Hsin-Cheng", family = "Huang", role = "aut",
           comment = c(ORCID = "0000-0002-5613-349X")),
    person(given = "Ching-Kang", family = "Ing", role = "aut",
           comment = c(ORCID = "0000-0003-1362-8246")))
Description: Constructs multi-resolution thin-plate spline basis
    functions on the sphere for use in spatial regression and large-scale
    spatial prediction problems. Implements the basis system described in
    Huang, Huang, and Ing (2025) "Multi-Resolution Spatial Methods on the
    Sphere: Efficient Prediction for Global Data", Environmetrics,
    <doi:10.1002/env.70092>. Heavy computations are written in 'C++' via
    'Rcpp' with optional 'OpenMP' parallelism.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 4.0)
Imports: Rcpp, RSpectra
LinkingTo: Rcpp, RcppEigen, RcppNumerical
Suggests: fields, testthat (>= 3.0.0)
SystemRequirements: GNU make
RoxygenNote: 7.3.2
URL: https://github.com/STLABTW/multi-resolution-sphere
BugReports: https://github.com/STLABTW/multi-resolution-sphere/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-28 23:07:49 UTC; hyhuang
Author: Hao-Yun Huang [aut, cre] (ORCID:
    <https://orcid.org/0009-0004-8384-3261>),
  Hsin-Cheng Huang [aut] (ORCID: <https://orcid.org/0000-0002-5613-349X>),
  Ching-Kang Ing [aut] (ORCID: <https://orcid.org/0000-0003-1362-8246>)
Maintainer: Hao-Yun Huang <hhuscout@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 18:40:07 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-29 20:32:20 UTC; unix
Archs: mrtsSphere.so.dSYM
