# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_Kf <- function(L1, l1, L2, l2) {
    .Call(`_mrtsSphere_cpp_Kf`, L1, l1, L2, l2)
}

cpp_K <- function(X, Y, n) {
    .Call(`_mrtsSphere_cpp_K`, X, Y, n)
}

cpp_Kmatrix6 <- function(KK, X, ggrids, Konev, eiKvecmval, n, N, aa_grid, tab_vals) {
    .Call(`_mrtsSphere_cpp_Kmatrix6`, KK, X, ggrids, Konev, eiKvecmval, n, N, aa_grid, tab_vals)
}

